var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
define(["require", "exports", "aurelia-framework", "date-fns", "popper.js", "./scope", "./configuration"], function (require, exports, aurelia_framework_1, date_fns_1, popper_js_1, scope_1, configuration_1) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var AuDateTimePicker = (function () {
        function AuDateTimePicker(element) {
            this.element = element;
            this.options = configuration_1.DEFAULT_CONFIGURATION;
            this.amPm = 'AM';
            this.mode = 'date';
            this.scopes = scope_1.DEFAULT_SCOPES.array;
            this.currentScope = scope_1.DEFAULT_SCOPES.initial;
        }
        AuDateTimePicker.prototype.created = function () {
            this.initialize();
        };
        AuDateTimePicker.prototype.jQueryHas = function (element, target) {
            return element !== target && element.contains(target);
        };
        AuDateTimePicker.prototype.attached = function () {
            var reference = this.element.querySelectorAll('.datepicker-input').item(0);
            this.poppedElement = this.element.querySelectorAll('.dropdown-menu').item(0);
            var self = this;
            this.hideCallback = function (event) {
                if (self.poppedElement !== event.target &&
                    !self.jQueryHas(self.poppedElement, event.target) &&
                    reference !== event.target) {
                    self.hide();
                }
            };
            document.getElementsByTagName('body').item(0).addEventListener('click', this.hideCallback);
            this.popper = new popper_js_1.default(reference, this.poppedElement, {
                placement: 'auto-start'
            });
        };
        AuDateTimePicker.prototype.detached = function () {
            document.getElementsByTagName('body').item(0).removeEventListener('click', this.hideCallback);
        };
        AuDateTimePicker.prototype.optionsChanged = function (newValue, oldValue) {
            if (!newValue) {
                this.options = configuration_1.DEFAULT_CONFIGURATION;
                this.scopes = configuration_1.DEFAULT_CONFIGURATION.scopes;
                this.currentScope = configuration_1.DEFAULT_CONFIGURATION.initialScope;
            }
            if (newValue !== oldValue) {
                console.error('Not yet implemented');
            }
        };
        AuDateTimePicker.prototype.selectedDateChanged = function (newValue, oldValue) {
            if (isNaN(Date.parse(newValue)) && newValue !== null) {
                throw new Error('Datetimepicker, model.bind must be of type Date');
            }
            if (!newValue) {
                newValue = new Date();
            }
            if (newValue !== oldValue && newValue) {
                this.textValue = this.formatDate(this.selectedDate, this.options.format);
            }
        };
        AuDateTimePicker.prototype.textValueChanged = function (newValue, oldValue) {
            if (newValue !== oldValue && newValue) {
                this.selectedDate = date_fns_1.parse(newValue, this.options.format, new Date());
            }
        };
        AuDateTimePicker.prototype.initialize = function () {
            this.now = this.viewedDate = this.selectedDate = new Date();
        };
        AuDateTimePicker.prototype.getDaysOfWeek = function () {
            var _this = this;
            var firstDay = this.getFirstDayOfWeek(new Date());
            var daysOfWeek = Array.from(new Array(7), function (_, index) { return date_fns_1.addDays(firstDay, index); })
                .map(function (date) { return _this.formatDate(date, 'dd'); });
            return daysOfWeek;
        };
        AuDateTimePicker.prototype.getOptions = function (currentScope, date) {
            return currentScope.getItems(date);
        };
        AuDateTimePicker.prototype.getFirstDayOfWeek = function (date) {
            return date_fns_1.startOfWeek(date, { weekStartsOn: this.options.weekStartsOn });
        };
        AuDateTimePicker.prototype.next = function () {
            this.viewedDate = this.currentScope.next(this.viewedDate);
        };
        AuDateTimePicker.prototype.previous = function () {
            this.viewedDate = this.currentScope.previous(this.viewedDate);
        };
        AuDateTimePicker.prototype.selectItem = function (item) {
            this.viewedDate = item;
            if (this.currentScope.isFinal()) {
                this.selectedDate = item;
            }
            else {
                this.currentScope = this.currentScope.nextScope();
            }
        };
        AuDateTimePicker.prototype.isSelected = function (date) {
            return this.currentScope.isEqual(date, this.selectedDate);
        };
        AuDateTimePicker.prototype.isDisplayed = function (date) {
            return !this.currentScope.isFinal() &&
                this.currentScope.isEqual(date, this.viewedDate);
        };
        AuDateTimePicker.prototype.isNow = function (date) {
            return this.currentScope.isEqual(date, this.now);
        };
        AuDateTimePicker.prototype.showDays = function () {
            return this.currentScope.isFinal();
        };
        AuDateTimePicker.prototype.getLabel = function (item) {
            return this.currentScope.formatDate(item, this.locale);
        };
        AuDateTimePicker.prototype.zoomOut = function () {
            var previousStrategy = this.currentScope.previousScope();
            if (previousStrategy != null) {
                this.currentScope = previousStrategy;
                this.viewedDate = new Date(this.viewedDate);
            }
        };
        AuDateTimePicker.prototype.formatDate = function (date, format) {
            return date_fns_1.format(date, format, { locale: this.locale });
        };
        AuDateTimePicker.prototype.getTitle = function (date) {
            return this.currentScope.title(date, this.locale);
        };
        AuDateTimePicker.prototype.getItemClasses = function (date) {
            var classes = [];
            if (this.isDisplayed(date)) {
                classes.push('displayed');
            }
            if (this.isSelected(date)) {
                classes.push('selected');
            }
            if (this.isNow(date)) {
                classes.push('now');
            }
            if (this.currentScope.isFinal()) {
                classes.push('picker-small');
            }
            else {
                classes.push('picker-large');
            }
            return classes.join(' ');
        };
        AuDateTimePicker.prototype.toggleAmPm = function () {
            if (this.amPm === 'AM') {
                this.selectedDate = date_fns_1.addHours(this.selectedDate, 12);
                this.amPm = 'PM';
            }
            else {
                this.selectedDate = date_fns_1.addHours(this.selectedDate, -12);
                this.amPm = 'AM';
            }
        };
        AuDateTimePicker.prototype.toggleMode = function () {
            this.mode = this.mode === 'date' ? 'time' : 'date';
        };
        AuDateTimePicker.prototype.addHours = function (hours) {
            this.selectedDate = date_fns_1.addHours(this.selectedDate, hours);
        };
        AuDateTimePicker.prototype.addMinutes = function (minutes) {
            this.selectedDate = date_fns_1.addMinutes(this.selectedDate, minutes);
        };
        AuDateTimePicker.prototype.selectHours = function () {
            console.error('Not yet implemented');
        };
        AuDateTimePicker.prototype.selectMinutes = function () {
            console.error('Not yet implemented');
        };
        AuDateTimePicker.prototype.isVisible = function () {
            return this.poppedElement.style.display === '';
        };
        AuDateTimePicker.prototype.show = function () {
            this.poppedElement.style.display = '';
        };
        AuDateTimePicker.prototype.hide = function () {
            this.poppedElement.style.display = 'none';
        };
        AuDateTimePicker.prototype.inputFocus = function () {
            if (this.options.showOnInputFocus) {
                this.show();
            }
        };
        AuDateTimePicker.prototype.toggle = function () {
            this.isVisible() ? this.hide() : this.show();
        };
        __decorate([
            aurelia_framework_1.bindable({ defaultBindingMode: aurelia_framework_1.bindingMode.twoWay }),
            __metadata("design:type", Date)
        ], AuDateTimePicker.prototype, "selectedDate", void 0);
        __decorate([
            aurelia_framework_1.bindable({ defaultBindingMode: aurelia_framework_1.bindingMode.twoWay }),
            __metadata("design:type", String)
        ], AuDateTimePicker.prototype, "textValue", void 0);
        __decorate([
            aurelia_framework_1.bindable,
            __metadata("design:type", Object)
        ], AuDateTimePicker.prototype, "options", void 0);
        AuDateTimePicker = __decorate([
            aurelia_framework_1.autoinject(),
            __metadata("design:paramtypes", [Element])
        ], AuDateTimePicker);
        return AuDateTimePicker;
    }());
    exports.AuDateTimePicker = AuDateTimePicker;
});
